
#ifndef _SUBVECTOR_H_
#define _SUBVECTOR_H_ 1

#include "typedefs.h"

// Some internal functions to deal with vectors and matrices
// Copy entries starting at 'start' and ending in 'stop'-1 position
// Caution: entry or row on 'stop' position not coppied!
using capd::vectalg::Vector;
using capd::vectalg::Matrix;

template <typename T, int dimSrc, int dimTar>
void copyToSubvector(const capd::vectalg::Vector<T, dimSrc> &src, 
                     capd::vectalg::Vector<T, dimTar> &target, 
                     int start, int stop)
{
  for(int iTarget=start, iSrc=0; iTarget<stop; ++iTarget, ++iSrc)
    target[iTarget] = src[iSrc];

}

template <typename T, int rowSrc, int colSrc, int rowTar, int colTar>
void copyToSubmatrix(const capd::vectalg::Matrix<T,rowSrc, colSrc> &src, 
                     capd::vectalg::Matrix<T, rowTar, colTar> &target, 
                     int start, int stop)
{
  for(int iTarget=start, iSrc=0; iTarget<stop; ++iTarget, ++iSrc)
    for(int jTarget=start, jSrc=0; jTarget<stop; ++jTarget, ++jSrc)
      target[iTarget][jTarget] = src[iSrc][jSrc];
}

template <typename T, int dimSrc, int dimTar>
void copyFromSubvector(const capd::vectalg::Vector<T, dimSrc> &src, 
                       capd::vectalg::Vector<T, dimTar> &target, 
                       int start, int stop)
{
  for(int iSrc=start, iTarget=0; iSrc<stop; ++iTarget, ++iSrc)
    target[iTarget] = src[iSrc];

}

template <typename T, int rowSrc, int colSrc, int rowTar, int colTar>
void copyFromSubmatrix(const capd::vectalg::Matrix<T,rowSrc, colSrc> &src, 
                     capd::vectalg::Matrix<T, rowTar, colTar> &target, 
                     int startRow, int stopRow, 
                     int startCol = -1, int stopCol = -1)
{
  if(startCol == -1)
    startCol = startRow;
  if(stopCol == -1)
    stopCol = stopRow;

  for(int iSrc=startRow, iTarget=0; iSrc<stopRow; ++iTarget, ++iSrc)
    for(int jSrc=startCol, jTarget=0; jSrc<stopCol; ++jTarget, ++jSrc)
      target[iTarget][jTarget] = src[iSrc][jSrc];
}

template <typename T, int dimSrc, int dimTar>
void intersectionWithSubvector(const capd::vectalg::Vector<T, dimSrc> &src, 
                               capd::vectalg::Vector<T, dimTar> &target, 
                               int start, int stop)
{
  int result = 1;
  for(int iTarget=start, iSrc=0; iTarget<stop; ++iTarget, ++iSrc)
    result = (result && intersection (src[iSrc], target[iTarget], target[iTarget]));
  if(!result)
    throw std::runtime_error(" Null intersection in intersectionWithSubvector ");
}

template <typename T, int rowSrc, int colSrc, int rowTar, int colTar>
void intersectionWithSubmatrix(const capd::vectalg::Matrix<T,rowSrc, colSrc> &src, 
                               capd::vectalg::Matrix<T, rowTar, colTar> &target, 
                               int start, int stop)

{
  int result =  1;
  for(int iTarget=start, iSrc=0; iTarget<stop; ++iTarget, ++iSrc)
    for(int jTarget=start, jSrc=0; jTarget<stop; ++jTarget, ++jSrc)
      result = (result && intersection(src[iSrc][jSrc], target[iTarget][jTarget], target[iTarget][jTarget]));
  if(!result)
    throw std::runtime_error(" Null intersection in intersectionWithSubvector ");
}

#endif //_SUBVECTOR_H_

