#include <string>
#include <sstream>

std::string onePart3D(int index,int numberOfBodies)
{
   std::ostringstream result;
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 1 << "-q" << index <<"1)";
      result << "/sqr("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2+"
             << "(q" << i << "3-q" << index << "3)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   result << ",";
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 2 << "-q" << index <<"2)";
      result << "/sqr("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2+"
             << "(q" << i << "3-q" << index << "3)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   result << ",";
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 3 << "-q" << index <<"3)";
      result << "/sqr("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2+"
             << "(q" << i << "3-q" << index << "3)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   return result.str();
}

std::string nBodyEquation3D(int numberOfBodies)
{
   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ","
          << "q" << i << "3" << ",";
   }
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ","
          << "p" << i << "3";
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }

   var << "fun:";
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ","
          << "p" << i << "3" << ",";
   }

   for(i=1;i<=numberOfBodies;i++)
   {
      var << onePart3D(i,numberOfBodies);
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }
   return var.str();
}


std::string oneParticle2D(int index,int numberOfBodies)
{
   std::ostringstream result;
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 1 << "-q" << index <<"1)";
      result << "/sqr("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   result << ",";
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 2 << "-q" << index <<"2)";
      result << "/sqr("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   return result.str();
}

std::string nBodyEquation2D(int numberOfBodies)
{
   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ","
          << "p" << i << "1" << ","
          << "p" << i << "2";
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }

   var << "fun:";

   for(i=1;i<=numberOfBodies;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ",";
      var << oneParticle2D(i,numberOfBodies);
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }
   return var.str();
}


/*************************************************************************/
//
//    Planar N-body Equation reduced by center of the mass condition
//
//    variables are
//      (q11,q12,p11,p12,q21,q22,...,q(N-1)1,q(N-1)2,p(N-1)1,p(N-1)2)
//
//    q - positions,  p - velocities
//
/*************************************************************************/


// It gives  'coord' coordinate of the last particle computed from the center of mass condition
// if coord is greater than 2 then we put velocities coordinates.

std::string lastParticle(int coord, int numberOfBodies)
{
   std::ostringstream result;
   int i;
   char leter = 'q';
   if(coord>=3)
   {
       coord-=2;
       leter = 'p';
   }

   result <<"(";
   for(i=1; i<=numberOfBodies-1; ++i)
   {
     result << "-" << leter << i << coord;
   }
   result << ")";

   return result.str();
}


std::string reducedOneParticle2D(int index,int numberOfBodies)
{
   std::ostringstream result;

   std::string qN[2];
   qN[0] = lastParticle(1, numberOfBodies);
   qN[1] = lastParticle(2, numberOfBodies);

   int i, j;
   for(j=1; j<=2; ++j)
   {
      for(i=1;i<=numberOfBodies-1; ++i)
      {
         if(i==index) continue;
         result << "(q" << i << j <<"-q" << index << j <<")";
         result << "*(("
                << "(q" << i << "1-q" << index << "1)*(q" << i << "1-q" << index << "1)+"
                << "(q" << i << "2-q" << index << "2)*(q" << i << "2-q" << index << "2)"
                << ")^(-1.5))";
         result << "+";
      }
      // Last body
      result << "(" << qN[j-1] <<"-q"<< index << j;
      result << ")*((";
      result << "(q" << index << "1-" << qN[0] << ")*(q" << index << "1-" << qN[0] << ")+"
             << "(q" << index << "2-" << qN[1] << ")*(q" << index << "2-" << qN[1] << ")"
             << ")^(-1.5))";

      if (j==1) result << ",";
   }

   return result.str();
}

/*********************************************************************
  Generates 2D N-body equation reduced by center of mass condition
*********************************************************************/

std::string reducedNBodyEquation2D(int numberOfBodies)
{
   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies-1;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ",";
      var << "p" << i << "1" << ","
          << "p" << i << "2";
      if(i!=numberOfBodies-1)
         var << ",";
      else
         var << ";";
   }

   var << "fun:";
   for(i=1;i<=numberOfBodies-1;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ",";
      var << reducedOneParticle2D(i,numberOfBodies);

      if(i!=numberOfBodies-1)
         var << ",";
      else
         var << ";";
   }
   return var.str();
}


//******************************************************************************/
// Generates First Integrals Function (angular momentum, total energy)
//   for system after 'center of the mass' reduction
//******************************************************************************/

std::string reducedKineticEnergy(int numberOfBodies)
{
   std::ostringstream var;
   std::string    pN1 = lastParticle(3, numberOfBodies),
                  pN2 = lastParticle(4, numberOfBodies);

   for(int i=1; i<=numberOfBodies-1; i++)
   {
      var << "("
          << "p" << i <<"1*p" << i<<"1+"
          << "p" << i <<"2*p" << i<<"2"
          << ")/2+";
   }

   // Last body
   var << "("
       << pN1 <<"*" << pN1 << "+"
       << pN2 <<"*" << pN2
       << ")/2";

   return var.str();
}


std::string reducedPotentialEnergy(int numberOfBodies)
{
   std::ostringstream var;

   std::string   qN1 = lastParticle(1, numberOfBodies),
                 qN2 = lastParticle(2, numberOfBodies);

   var << "(";
   for(int i=1; i<=numberOfBodies-1; ++i)
   {
      for(int j=i+1; j<numberOfBodies; ++j)
      {
         var << "(("
             << "(q" << j << "1-q" << i << "1)*(q" << j << "1-q" << i << "1)+"
             << "(q" << j << "2-q" << i << "2)*(q" << j << "2-q" << i << "2)"
             << ")^(-0.5))+";
      }
      // Last body
      var << "(("
          << "(" << qN1 << "-q" << i << "1)*(" << qN1 << "-q" << i << "1)+"
          << "(" << qN2 << "-q" << i << "2)*(" << qN2 << "-q" << i << "2)"
          << ")^(-0.5))";

      if (i!=numberOfBodies-1)
         var << "+";
   }
   var <<")";

   return var.str();
}


std::string reducedAngularMomentum(int numberOfBodies)
{
   std::ostringstream var;

   std::string     qN1 = lastParticle(1, numberOfBodies),
                   qN2 = lastParticle(2, numberOfBodies),
                   pN1 = lastParticle(3, numberOfBodies),
                   pN2 = lastParticle(4, numberOfBodies);

    // Angular momentum - planar case

   var << "(";
   for(int i=1; i<numberOfBodies; i++)
   {
      var << "q" << i << "1*p" << i << "2-"
          << "q" << i << "2*p" << i << "1+";
   }
   var << qN1 << "*" << pN2 << "-"
       << qN2 << "*" << pN1 << ")";

   return var.str();

}

std::string reducedFirstIntegrals2D(int numberOfBodies)
{

   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies-1;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ",";
      var << "p" << i << "1" << ","
          << "p" << i << "2";
      if(i!=numberOfBodies-1)
         var << ",";
      else
         var << ";";
   }

   var << "fun:" << reducedKineticEnergy(numberOfBodies)
       << "-" << reducedPotentialEnergy(numberOfBodies) << ","
       << reducedAngularMomentum(numberOfBodies);

   for(i=0; i<((4*numberOfBodies)-6); ++i)
      var << ",0";
   var << ";" << std::ends;

   return var.str();
}
