

#include <sstream>
#include <iomanip>
#include <stdexcept>


//#include "dimen.h"
#include "typedefs.h"
#include "nbody.h"
#include "mapping.h"

namespace capd{

typedef newton::Mapping<IntervalMatrix> Mapping;
//*****************************************************************************/
/*   PROOF  - Rigorous computation   -  we use Krawczyk metod                 */
//   to prove existence of zero of funtion F(x)
//
//   INPUT:
//     x    - approximated zero of F,
//     size - radius of set X,  ( X = x + [-size, size]^n )
// or  X    - set X
//     F    - class which can calculate value of the function F and its derivative dF
//
//   We compute Krawczyk operator
//      K = x - F(x) + (Id - dF(X))(X-x0)
//
//   OUTPUT
//     1 - zero exists (K is subset of int X),
//     0 - there is no zeros in the set X (intersection of K and X is empty),
//    -1 - we can still take intersection of X and K, but we exceed maximal number of iteration
//    -2 - X is subset of K,
/*****************************************************************************/

int KrawczykProof (FloatVector &x,  double size,  Mapping &F);
int KrawczykProof (IntervalVector x0,  IntervalVector X,  Mapping &F);

/**************************************************************************************/
//   Computes Krawczyk operator
//
//   INPUT:
//     x    - approximated zero of F,
//     X    - set X
//     F    - class which can calculate value of the function F and its derivative dF
//
//   OUTPUT:
//      K = x - F(x) + (Id - dF(X))(X-x0)
//
/*************************************************************************************/

IntervalVector KrawczykOperator (IntervalVector x0,  IntervalVector X,  Mapping &F);


}  //end namespace capd


